@@ -2,7 +2,7 @@
 
 from weakref import finalize, WeakValueDictionary
 from sqlite3 import OperationalError, connect
-from typing import Iterable, Optional, Mapping
+from typing import Iterable, Literal, Optional, Mapping
 
 from .locals import PLUGINS_PATH
 from ._utils import (
@@ -82,9 +82,12 @@ class Database:
             cursor.execute(query)
             self._database.commit()
         except OperationalError as error:
-            dberror = DatabaseExistsError(f"table {table} already exists.")
-            dberror.add_note(f"{type(error).__name__}: {error!s}")
-            raise dberror from error
+            if "already exists" in str(error):
+                dberror = DatabaseExistsError(f"table {table} already exists.")
+                dberror.add_note(f"{type(error).__name__}: {error!s}")
+                raise dberror from error
+            error.add_note(f"Query: {query}")
+            raise error
         table_ = self.table(table, columns)
         table_._deleted = False  # pylint: disable=protected-access
         self._table_instances[table] = table_
@@ -175,6 +178,32 @@ class Database:
                 listed.append(self.table(table.name))
         return tuple(listed)
 
+    def commit(self):
+        """Commit changes to database"""
+        self._database.commit()
+
+    def rollback(self):
+        """Rollback changes"""
+        self._database.rollback()
+
+    def foreign_pragma(self, bool_state: Literal['ON', "OFF", ""] = ""):
+        """Enable/disable foreign key pragma"""
+        if bool_state not in ("ON", "OFF", ""):
+            raise ValueError("Either ON/OFF for foreign key pragma.")
+        return self._database.execute(f"PRAGMA foreign_keys{'='+bool_state  if bool_state else ''}").fetchone() # pylint: disable=line-too-long
+
+    def optimize(self):
+        """Optimize current database"""
+        return self._database.execute("PRAGMA optimize").fetchone()
+
+    def shrink_memory(self):
+        """Shrink memories from database as much as it can."""
+        return self._database.execute("PRAGMA shrink_memory").fetchone()
+
+    def vacuum(self):
+        """Vacuum this database"""
+        return self._database.execute("VACUUM").fetchone()
+
     @property
     def closed(self):
         """Is database closed?"""
